@@ -1,5 +1,6 @@
 
 #include <getopt.h>
+#include <signal.h>
 #include <stdio.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
@@ -145,7 +146,7 @@ static int cmd_data_job(int argc, char *argv[])
 static int data_usage(void)
 {
 	puts("bcachefs data - manage filesystem data\n"
-	     "Usage: bcachefs data <CMD> [OPTIONS]\n"
+	     "Usage: bcachefs data <rereplicate|scrub|job> [OPTION]...\n"
 	     "\n"
 	     "Commands:\n"
 	     "  rereplicate                  Rereplicate degraded data\n"
@@ -153,7 +154,7 @@ static int data_usage(void)
 	     "  job                          Kick off low level data jobs\n"
 	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
-	return 0;
+	exit(EXIT_SUCCESS);
 }
 
 int data_cmds(int argc, char *argv[])
@@ -177,10 +178,8 @@ int data_cmds(int argc, char *argv[])
 
 static void scrub_usage(void)
 {
-	puts("bcachefs scrub\n"
-	     "Usage: bcachefs scrub [filesystem|device]\n"
-	     "\n"
-	     "Check data for errors, fix from another replica if possible\n"
+	puts("bcachefs scrub - verify checksums and correct errors, if possible\n"
+	     "Usage: bcachefs scrub <filesystem|device>\n"
 	     "\n"
 	     "Options:\n"
 	     "  -m, --metadata               Check metadata only\n"
@@ -389,11 +388,9 @@ static void reconcile_wait_usage(void)
 	CLASS(printbuf, buf)();
 	prt_bitflags(&buf, __bch2_reconcile_accounting_types, ~0UL);
 
-	printf("bcachefs reconcile wait\n"
+	printf("bcachefs reconcile wait - wait for reconcile to finish background data processing\n"
 	     "Usage: bcachefs reconcile wait [OPTION]... <mountpoint>\n"
 	     "\n"
-	     "Wait for reconcile to finish background data processing of one or more types\n"
-	     "\n"
 	     "Options:\n"
 	     "  -t, --types=TYPES            List of reconcile types to wait on\n"
 	     "                               %s\n"
@@ -410,7 +407,7 @@ static bool reconcile_status(struct printbuf *out,
 {
 	bool scan_pending = read_file_u64(fs.sysfs_fd, "reconcile_scan_pending");
 
-	u64 v[BCH_REBALANCE_ACCOUNTING_NR][2];
+	u64 v[BCH_RECONCILE_ACCOUNTING_NR][2];
 	memset(v, 0, sizeof(v));
 
 	struct bch_ioctl_query_accounting *a =
@@ -467,6 +464,14 @@ static size_t count_newlines(const char *str)
 	return ret;
 }
 
+static const char *restore_screen_str = "\033[?1049l";
+
+static void exit_restore_screen(int n)
+{
+	write(STDOUT_FILENO, restore_screen_str, strlen(restore_screen_str));
+	exit(EXIT_SUCCESS);
+}
+
 int cmd_reconcile_wait(int argc, char *argv[])
 {
 	static const struct option longopts[] = {
@@ -474,7 +479,7 @@ int cmd_reconcile_wait(int argc, char *argv[])
 		{ "help",		no_argument,		NULL, 'h' },
 		{ NULL }
 	};
-	unsigned types = ~0U & ~BIT(BCH_REBALANCE_ACCOUNTING_pending);
+	unsigned types = ~0U & ~BIT(BCH_RECONCILE_ACCOUNTING_pending);
 	int opt;
 
 	while ((opt = getopt_long(argc, argv, "t:h", longopts, NULL)) != -1)
@@ -499,6 +504,11 @@ int cmd_reconcile_wait(int argc, char *argv[])
 
 	write_file_str(fs.sysfs_fd, "internal/trigger_reconcile_wakeup", "1");
 
+	struct sigaction act = { .sa_handler = exit_restore_screen };
+	sigaction(SIGINT, &act, NULL);
+
+	fputs("\033[?1049h", stdout);
+
 	while (true) {
 		CLASS(printbuf, buf)();
 		bool pending = reconcile_status(&buf, fs, types);
@@ -512,6 +522,8 @@ int cmd_reconcile_wait(int argc, char *argv[])
 		sleep(1);
 	}
 
+	fputs(restore_screen_str, stdout);
+
 	return 0;
 }
 
@@ -520,7 +532,7 @@ static void reconcile_status_usage(void)
 	CLASS(printbuf, buf)();
 	prt_bitflags(&buf, __bch2_reconcile_accounting_types, ~0UL);
 
-	printf("bcachefs reconcile status\n"
+	printf("bcachefs reconcile status - show the status of a background reconciliation processing\n"
 	       "Usage: bcachefs reconcile status [OPTION]... <mountpoint>\n"
 	       "\n"
 	       "Options:\n"
@@ -573,14 +585,14 @@ int cmd_reconcile_status(int argc, char *argv[])
 static int reconcile_usage(void)
 {
 	puts("bcachefs reconcile - manage data reconcile\n"
-	     "Usage: bcachefs reconcile <CMD> [OPTIONS]\n"
+	     "Usage: bcachefs reconcile <status|wait> [OPTION]...\n"
 	     "\n"
 	     "Commands:\n"
 	     "  status                       Show status of background data processing\n"
 	     "  wait                         Wait on background data processing to complete\n"
 	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
-	return 0;
+	exit(EXIT_SUCCESS);
 }
 
 int reconcile_cmds(int argc, char *argv[])
