@@ -1,5 +1,8 @@
 
+#include <asm/types.h>
+
 #include <errno.h>
+#include <fcntl.h>
 #include <getopt.h>
 #include <sys/uio.h>
 #include <unistd.h>
