@@ -16,11 +16,10 @@
 #include "btree/write_buffer.h"
 
 #include "data/compress.h"
-#include "data/ec.h"
 #include "data/keylist.h"
 #include "data/move.h"
 #include "data/read.h"
-#include "data/reconcile.h"
+#include "data/reconcile/trigger.h"
 #include "data/reflink.h"
 #include "data/write.h"
 
@@ -207,8 +206,8 @@ static int __bch2_move_extent(struct moving_context *ctxt,
 		     struct move_bucket *bucket_in_flight,
 		     struct btree_iter *iter,
 		     struct bkey_s_c k,
-		     struct bch_inode_opts io_opts,
-		     struct data_update_opts data_opts)
+		     struct bch_inode_opts *io_opts,
+		     struct data_update_opts *data_opts)
 {
 	struct btree_trans *trans = ctxt->trans;
 	struct bch_fs *c = trans->c;
@@ -225,7 +224,7 @@ static int __bch2_move_extent(struct moving_context *ctxt,
 		return ret;
 
 	ret = bch2_data_update_init(trans, iter, ctxt, u, ctxt->wp,
-				    &io_opts, data_opts, iter->btree_id, k);
+				    io_opts, *data_opts, iter->btree_id, k);
 	if (ret)
 		return bch2_err_matches(ret, BCH_ERR_data_update_done) ? 0 : ret;
 
@@ -266,8 +265,8 @@ static int __bch2_move_extent(struct moving_context *ctxt,
 			   bkey_start_pos(k.k),
 			   iter->btree_id, k, 0,
 			   NULL,
-			   data_opts.read_flags|BCH_READ_last_fragment,
-			   data_opts.read_dev);
+			   data_opts->read_flags|BCH_READ_last_fragment,
+			   data_opts->read_dev);
 	u = NULL;
 	return 0;
 }
@@ -287,7 +286,7 @@ int bch2_move_extent(struct moving_context *ctxt,
 		return bch_err_throw(c, device_offline);
 
 	if (!bkey_is_btree_ptr(k.k))
-		ret = __bch2_move_extent(ctxt, bucket_in_flight, iter, k, *opts, *data_opts);
+		ret = __bch2_move_extent(ctxt, bucket_in_flight, iter, k, opts, data_opts);
 	else if (data_opts->type != BCH_DATA_UPDATE_scrub) {
 		if (data_opts->type != BCH_DATA_UPDATE_copygc)
 			try(bch2_can_do_data_update(trans, opts, data_opts, k, NULL));
@@ -332,7 +331,7 @@ static int bch2_move_extent_pred(struct moving_context *ctxt,
 	struct bch_inode_opts opts;
 	try(bch2_bkey_get_io_opts(trans, snapshot_io_opts, k, &opts));
 	try(bch2_update_reconcile_opts(trans, snapshot_io_opts, &opts, iter, level, k,
-				       SET_NEEDS_REBALANCE_other));
+				       SET_NEEDS_RECONCILE_other));
 
 	CLASS(disk_reservation, res)(c);
 	try(bch2_trans_commit_lazy(trans, &res.r, NULL, BCH_TRANS_COMMIT_no_enospc));
